@@ -13,6 +13,7 @@ MYDIR=$(dirname $(realpath "$0"))
 
 # Git revisions we use for the given submodules. Update these whenever you
 # update a git submodule.
+TESTDATA="873045a9c42ed60721756e26e2a6b32e17415205"
 THIRD_PARTY_BROTLI="36533a866ed1ca4b75cf049f4521e4ec5fe24727"
 THIRD_PARTY_HIGHWAY="ba0900a4957b929390ab73827235557959234fea"
 THIRD_PARTY_SKCMS="42030a771244ba67f86b1c1c76a6493f873c5f91"
@@ -27,7 +28,7 @@ download_github() {
   local project="$2"
 
   local varname=`echo "$path" | tr '[:lower:]' '[:upper:]'`
-  varname="${varname/[\/-]/_}"
+  varname="${varname//[\/-]/_}"
   local sha
   eval "sha=\${${varname}}"
 
@@ -79,6 +80,7 @@ EOF
   fi
 
   # Sources downloaded from a tarball.
+  download_github testdata libjxl/testdata
   download_github third_party/brotli google/brotli
   download_github third_party/highway google/highway
   download_github third_party/sjpeg webmproject/sjpeg
