@@ -8,14 +8,12 @@
 
 // Interleaved image for color transforms and Codec.
 
+#include <jxl/types.h>
 #include <stddef.h>
 #include <stdint.h>
 
-#include "jxl/types.h"
 #include "lib/jxl/base/data_parallel.h"
-#include "lib/jxl/base/padded_bytes.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/codec_in_out.h"
 #include "lib/jxl/color_encoding_internal.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/image_bundle.h"
